/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QFont>
#include <QFrame>
#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QRadioButton>
#include <QDoubleValidator>
#include <QSpacerItem>
#include <QResizeEvent>
#include <QScreen>
#include <QSize>
#include <QDir>
#include <QCloseEvent>
#include <QSettings>

#include "tempcomp.h"

TTempComp::TTempComp(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QScreen								*screen;
	QFont								copyright_font;
	QFont								length_font;
	QFrame								*lower_separator_line;
	QFrame								*middle_separator_line;
	QFrame								*upper_separator_line;
	QLabel								*axis_cte_label;
	QLabel								*axis_temp_label;
	QLabel								*correction_label;
	QLabel								*expected_length_label;
	QLabel								*nominal_length_label;
	QLabel								*part_cte_label;
	QLabel								*part_temp_label;
	QLabel								*units_label;
	QLabel								*information_label;
	QSpacerItem							*control_vspacer_upper;
	QSpacerItem							*control_vspacer_lower;
	QSpacerItem							*control_hspacer;
	QDir								d_settings_path;

	copyright_font.setPointSize(8);
	copyright_font.setItalic(true);
	
	length_font.setStretch(105);
	length_font.setWeight(QFont::Medium);	// Medium DemiBold
	
	d_settings_path = QDir::home();

	if(!d_settings_path.exists(".TempComp"))
	{
		d_settings_path.mkdir(".TempComp");
	}
	
	d_settings_path.cd(".TempComp");
	d_settings = new QSettings(d_settings_path.absolutePath() + "/settings.ini",QSettings::IniFormat);
	
	this->resize(851,585);

	d_dialog_layout = new QGridLayout(this);

	d_icon_label = new QLabel(this);
	d_icon_label->setMinimumSize(359,504);
	d_icon_label->setPixmap(QPixmap(QString::fromUtf8(":/gui/background.png")));
	d_icon_label->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
	d_dialog_layout->addWidget(d_icon_label,0,0,1,1);

	d_control_layout = new QGridLayout();

	units_label = new QLabel(this);
	d_control_layout->addWidget(units_label,0,0,1,1);

	d_metric_radio = new QRadioButton(this);
	d_control_layout->addWidget(d_metric_radio,0,1,1,1);

	d_imperial_radio = new QRadioButton(this);
	d_control_layout->addWidget(d_imperial_radio,1,1,1,1);

	upper_separator_line = new QFrame(this);
	upper_separator_line->setFrameShape(QFrame::HLine);
	upper_separator_line->setFrameShadow(QFrame::Sunken);
	d_control_layout->addWidget(upper_separator_line,2,0,1,3);

	axis_temp_label = new QLabel(this);
	d_control_layout->addWidget(axis_temp_label,3,0,1,1);

	d_axis_temp_edit = new QLineEdit(this);
	d_axis_temp_edit->setAlignment(Qt::AlignCenter);
	d_control_layout->addWidget(d_axis_temp_edit,3,1,1,1);

	d_axis_temp_units_label = new QLabel(this);
	d_control_layout->addWidget(d_axis_temp_units_label,3,2,1,1);

	axis_cte_label = new QLabel(this);
	d_control_layout->addWidget(axis_cte_label,4,0,1,1);

	d_axis_cte_edit = new QLineEdit(this);
	d_axis_cte_edit->setAlignment(Qt::AlignCenter);
	d_control_layout->addWidget(d_axis_cte_edit,4,1,1,1);

	d_axis_cte_units_label = new QLabel(this);
	d_control_layout->addWidget(d_axis_cte_units_label,4,2,1,1);

	part_temp_label = new QLabel(this);
	d_control_layout->addWidget(part_temp_label,5,0,1,1);

	d_part_temp_edit = new QLineEdit(this);
	d_part_temp_edit->setAlignment(Qt::AlignCenter);
	d_control_layout->addWidget(d_part_temp_edit,5,1,1,1);

	d_part_temp_units_label = new QLabel(this);
	d_control_layout->addWidget(d_part_temp_units_label,5,2,1,1);

	part_cte_label = new QLabel(this);
	d_control_layout->addWidget(part_cte_label,6,0,1,1);
	
	d_part_cte_edit = new QLineEdit(this);
	d_part_cte_edit->setAlignment(Qt::AlignCenter);
	d_control_layout->addWidget(d_part_cte_edit,6,1,1,1);

	d_part_cte_units_label = new QLabel(this);
	d_control_layout->addWidget(d_part_cte_units_label,6,2,1,1);
	
	middle_separator_line = new QFrame(this);
	middle_separator_line->setFrameShape(QFrame::HLine);
	middle_separator_line->setFrameShadow(QFrame::Sunken);
	d_control_layout->addWidget(middle_separator_line,7,0,1,3);

	nominal_length_label = new QLabel(this);
	nominal_length_label->setFont(length_font);
	d_control_layout->addWidget(nominal_length_label,8,0,1,1);

	d_nominal_length_edit = new QLineEdit(this);
	d_nominal_length_edit->setAlignment(Qt::AlignCenter);
	d_nominal_length_edit->setFont(length_font);
	d_control_layout->addWidget(d_nominal_length_edit,8,1,1,1);

	d_nominal_length_units_label = new QLabel(this);
	d_nominal_length_units_label->setFont(length_font);
	d_control_layout->addWidget(d_nominal_length_units_label,8,2,1,1);
	
	correction_label = new QLabel(this);
	correction_label->setFont(length_font);
	d_control_layout->addWidget(correction_label,9,0,1,1);
	
	d_correction_edit = new QLineEdit(this);
	d_correction_edit->setReadOnly(true);
	d_correction_edit->setAlignment(Qt::AlignCenter);
	d_correction_edit->setFont(length_font);
	d_control_layout->addWidget(d_correction_edit,9,1,1,1);
	
	d_correction_units_label = new QLabel(this);
	d_correction_units_label->setFont(length_font);
	d_control_layout->addWidget(d_correction_units_label,9,2,1,1);

	expected_length_label = new QLabel(this);
	expected_length_label->setFont(length_font);
	d_control_layout->addWidget(expected_length_label,10,0,1,1);

	d_expected_length_edit = new QLineEdit(this);
	d_expected_length_edit->setReadOnly(true);
	d_expected_length_edit->setAlignment(Qt::AlignCenter);
	d_expected_length_edit->setFont(length_font);
	d_control_layout->addWidget(d_expected_length_edit,10,1,1,1);

	d_expected_length_units_label = new QLabel(this);
	d_expected_length_units_label->setFont(length_font);
	d_control_layout->addWidget(d_expected_length_units_label,10,2,1,1);
	
	lower_separator_line = new QFrame(this);
	lower_separator_line->setFrameShape(QFrame::HLine);
	lower_separator_line->setFrameShadow(QFrame::Sunken);
	d_control_layout->addWidget(lower_separator_line,11,0,1,3);
	
	control_vspacer_upper = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	d_control_layout->addItem(control_vspacer_upper,12,0,1,3);

	information_label = new QLabel(this);
	information_label->setWordWrap(true);
	d_control_layout->addWidget(information_label, 13, 0, 1, 3);
	
	control_vspacer_lower = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	d_control_layout->addItem(control_vspacer_lower,14,0,1,3);
	
	control_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	d_control_layout->addItem(control_hspacer,0,4,13,1);

	d_dialog_layout->addLayout(d_control_layout,0,1,1,1);

	d_copyright_label = new QLabel(this);
	d_copyright_label->setFont(copyright_font);
	d_dialog_layout->addWidget(d_copyright_label,1,0,1,2);

	QWidget::setTabOrder(d_metric_radio,d_imperial_radio);
	QWidget::setTabOrder(d_imperial_radio,d_axis_temp_edit);
	QWidget::setTabOrder(d_axis_temp_edit,d_axis_cte_edit);
	QWidget::setTabOrder(d_axis_cte_edit,d_part_temp_edit);
	QWidget::setTabOrder(d_part_temp_edit,d_part_cte_edit);
	QWidget::setTabOrder(d_part_cte_edit,d_nominal_length_edit);
	QWidget::setTabOrder(d_nominal_length_edit,d_correction_edit);
	QWidget::setTabOrder(d_correction_edit,d_expected_length_edit);
	
	// defaults
	d_display_mode = TTempComp::DISPLAY_WIDE;
	
	d_metric_radio->setChecked(d_settings->value("Units_Metric",true).toBool());
	
	if(!d_metric_radio->isChecked())
	{
		d_imperial_radio->setChecked(true);
	}

	d_axis_temp = d_settings->value("Axis_Temp",20.0).toDouble();
	d_axis_cte = d_settings->value("Axis_CTE",10.0).toDouble();
	d_part_temp = d_settings->value("Part_Temp",20.0).toDouble();
	d_part_cte = d_settings->value("Part_CTE",11.5).toDouble();
	d_nominal_length = d_settings->value("Nominal_Length",1000.0).toDouble();
	
	// input validation
	d_axis_temp_validator = new QDoubleValidator(this);
	d_axis_temp_validator->setBottom(-99.99);
	d_axis_temp_validator->setTop(99.99);
	d_axis_temp_validator->setDecimals(2);
	d_axis_temp_validator->setNotation(QDoubleValidator::StandardNotation);
	
	d_axis_cte_validator = new QDoubleValidator(this);
	d_axis_cte_validator->setBottom(0.0);
	d_axis_cte_validator->setTop(99.9);
	d_axis_cte_validator->setDecimals(1);
	d_axis_cte_validator->setNotation(QDoubleValidator::StandardNotation);
	
	d_part_temp_validator = new QDoubleValidator(this);
	d_part_temp_validator->setBottom(-99.99);
	d_part_temp_validator->setTop(99.99);
	d_part_temp_validator->setDecimals(2);
	d_part_temp_validator->setNotation(QDoubleValidator::StandardNotation);
	
	d_part_cte_validator = new QDoubleValidator(this);
	d_part_cte_validator->setBottom(0.0);
	d_part_cte_validator->setTop(99.9);
	d_part_cte_validator->setDecimals(1);
	d_part_cte_validator->setNotation(QDoubleValidator::StandardNotation);

	d_nominal_length_validator = new QDoubleValidator(this);
	d_nominal_length_validator->setNotation(QDoubleValidator::StandardNotation);

	d_axis_temp_edit->setValidator(d_axis_temp_validator);
	d_axis_cte_edit->setValidator(d_axis_cte_validator);
	d_part_temp_edit->setValidator(d_part_temp_validator);
	d_part_cte_edit->setValidator(d_part_cte_validator);
	d_nominal_length_edit->setValidator(d_nominal_length_validator);
	
	if(d_axis_temp < d_axis_temp_validator->bottom() ||
	   d_axis_temp > d_axis_temp_validator->top())
	{
		d_axis_temp = 20.0;
	}
	
	if(d_axis_cte < d_axis_temp_validator->bottom() ||
	   d_axis_cte > d_axis_temp_validator->top())
	{
		d_axis_cte = 10.0;
	}
	
	if(d_part_temp < d_axis_temp_validator->bottom() ||
	   d_part_temp > d_axis_temp_validator->top())
	{
		d_part_temp = 20.0;
	}
	
	if(d_part_cte < d_axis_temp_validator->bottom() ||
	   d_part_cte > d_axis_temp_validator->top())
	{
		d_part_cte = 10.0;
	}
	
	if(d_nominal_length > 99999.999)
	{
		d_axis_temp = 1000.0;
	}
	
#ifdef Q_OS_ANDROID
    this->setWindowFlags(Qt::Window);
    this->showFullScreen();
#endif
	
	this->setWindowTitle(QStringLiteral("Temperature Compensation Version 1.0"));

	units_label->setText(QStringLiteral("Units:"));
	d_metric_radio->setText(QStringLiteral("Metric"));
	d_imperial_radio->setText(QStringLiteral("Imperial"));
	axis_temp_label->setText(QStringLiteral("Axis Temperature:"));
	axis_cte_label->setText(QStringLiteral("Axis Expansion Coefficient"));
	part_temp_label->setText(QStringLiteral("Part Temperature:"));
	part_cte_label->setText(QStringLiteral("Part Expansion Coefficient:"));
	nominal_length_label->setText(QStringLiteral("Nominal Measurement Length:"));
	correction_label->setText(QStringLiteral("Correction:"));
	expected_length_label->setText(QStringLiteral("Expected Measurement Length:"));
	information_label->setText(QStringLiteral("<html><head/><body><p><span style=\" font-weight:600;\">Information:</span>  This utility calculates the expected measurement length from a nominal length knowing the temperature of the CMM axis and part and the expansion coefficient of the CMM axis and part.</p></body></html>"));
	d_copyright_label->setText(QStringLiteral("Copyright (C) 2019 Select Calibration Incorporated.  www.selectcalibration.ca"));
	
	this->Display_Metric(d_metric_radio->isChecked());
	
	screen = QGuiApplication::primaryScreen();
	screen->setOrientationUpdateMask(Qt::PortraitOrientation | Qt::LandscapeOrientation | Qt::InvertedPortraitOrientation | Qt::InvertedLandscapeOrientation);
	
	connect(screen, SIGNAL(orientationChanged(Qt::ScreenOrientation)),this, SLOT(Screen_Orientation_Changed(Qt::ScreenOrientation)));
	
	connect(d_metric_radio,SIGNAL(toggled(bool)),this,SLOT(Display_Metric(bool)));
	connect(d_axis_temp_edit,SIGNAL(editingFinished(void)),this,SLOT(Axis_Temp_Changed(void)));
	connect(d_axis_cte_edit,SIGNAL(editingFinished(void)),this,SLOT(Axis_CTE_Changed(void)));
	connect(d_part_temp_edit,SIGNAL(editingFinished(void)),this,SLOT(Part_Temp_Changed(void)));
	connect(d_part_cte_edit,SIGNAL(editingFinished(void)),this,SLOT(Part_CTE_Changed(void)));
	connect(d_nominal_length_edit,SIGNAL(editingFinished(void)),this,SLOT(Length_Changed(void)));
}

TTempComp::~TTempComp(void)
{
}

void TTempComp::closeEvent(
	QCloseEvent							*event)
{
	d_settings->setValue("Units_Metric", d_metric_radio->isChecked());
	d_settings->setValue("Axis_Temp", d_axis_temp);
	d_settings->setValue("Axis_CTE", d_axis_cte);
	d_settings->setValue("Part_Temp", d_part_temp);
	d_settings->setValue("Part_CTE", d_part_cte);
	d_settings->setValue("Nominal_Length", d_nominal_length);

	event->accept();
}


void TTempComp::resizeEvent(
	QResizeEvent						*event)
{
	QSize								size(event->size());
	
	if(size.width() > 0 && size.height() > 0)
	{
		if(size.width() > size.height())
		{
			if(d_display_mode != TTempComp::DISPLAY_WIDE)
			{
				d_display_mode = TTempComp::DISPLAY_WIDE;
				this->Update_Layout();
			}
		}
		else
		{
			if(d_display_mode != TTempComp::DISPLAY_TALL)
			{
				d_display_mode = TTempComp::DISPLAY_TALL;
				this->Update_Layout();
			}
		}
	}
	
	QDialog::resizeEvent(event);
}

void TTempComp::Screen_Orientation_Changed(
	Qt::ScreenOrientation				orientation)
{
	if(orientation == Qt::LandscapeOrientation ||
	   orientation == Qt::InvertedLandscapeOrientation)
	{
		if(d_display_mode != TTempComp::DISPLAY_WIDE)
		{
			d_display_mode = TTempComp::DISPLAY_WIDE;
			this->Update_Layout();
		}
	}
	else if(orientation == Qt::PortraitOrientation ||
			orientation == Qt::InvertedPortraitOrientation)
	{
		if(d_display_mode != TTempComp::DISPLAY_TALL)
		{
			d_display_mode = TTempComp::DISPLAY_TALL;
			this->Update_Layout();
		}
	}
}

void TTempComp::Display_Metric(
	bool								is_metric)
{
	double								axis_temp;
	double								axis_cte;
	double								part_temp;
	double								part_cte;
	double								length;
	
	axis_temp = d_axis_temp;
	axis_cte = d_axis_cte;
	part_temp = d_part_temp;
	part_cte = d_part_cte;
	length = d_nominal_length;

	if(is_metric)
	{
		d_nominal_length_validator->setBottom(0.000);
		d_nominal_length_validator->setTop(99999.999);
		d_nominal_length_validator->setDecimals(3);
		
		d_nominal_length_edit->setText(QString("%1").arg(length,0,'f',3));
	}
	else
	{
		d_nominal_length_validator->setBottom(0.00000);
		d_nominal_length_validator->setTop(3999.99999);
		d_nominal_length_validator->setDecimals(5);
		
		axis_temp *= 9.0;
		axis_temp /= 5.0;
		axis_temp += 32.0;
		
		part_temp *= 9.0;
		part_temp /= 5.0;
		part_temp += 32.0;
		
		axis_cte *= 5.0;
		axis_cte /= 9.0;
		
		part_cte *= 5.0;
		part_cte /= 9.0;
		
		length /= 25.4;
		
		d_nominal_length_edit->setText(QString("%1").arg(length,0,'f',5));
	}
	
	d_axis_temp_edit->setText(QString("%1").arg(axis_temp,0,'f',2));
	d_axis_cte_edit->setText(QString("%1").arg(axis_cte,0,'f',1));
	d_part_temp_edit->setText(QString("%1").arg(part_temp,0,'f',2));
	d_part_cte_edit->setText(QString("%1").arg(part_cte,0,'f',1));
	
	this->Set_Unit_Labels(is_metric);
	this->Update_Expected_Length(is_metric);
}

void TTempComp::Axis_Temp_Changed(void)
{
	double								dval;
	bool								is_metric(d_metric_radio->isChecked());
	
	dval = d_axis_temp_edit->text().toDouble();

	if(!is_metric)
	{
		dval -= 32.0;
		dval *= 5.0;
		dval /= 9.0;
	}
	
	d_axis_temp = dval;

	this->Update_Expected_Length(is_metric);
}

void TTempComp::Axis_CTE_Changed(void)
{
	double								dval;
	bool								is_metric(d_metric_radio->isChecked());
	
	dval = d_axis_cte_edit->text().toDouble();
	
	if(!is_metric)
	{
		dval *= 9.0;
		dval /= 5.0;
	}
	
	d_axis_cte = dval;
	
	this->Update_Expected_Length(is_metric);
}

void TTempComp::Part_Temp_Changed(void)
{
	double								dval;
	bool								is_metric(d_metric_radio->isChecked());
	
	dval = d_part_temp_edit->text().toDouble();
	
	if(!is_metric)
	{
		dval -= 32.0;
		dval *= 5.0;
		dval /= 9.0;
	}
	
	d_part_temp = dval;
	
	this->Update_Expected_Length(is_metric);
}

void TTempComp::Part_CTE_Changed(void)
{
	double								dval;
	bool								is_metric(d_metric_radio->isChecked());
	
	dval = d_part_cte_edit->text().toDouble();
	
	if(!is_metric)
	{
		dval *= 9.0;
		dval /= 5.0;
	}
	
	d_part_cte = dval;
	
	this->Update_Expected_Length(is_metric);
}

void TTempComp::Length_Changed(void)
{
	double								dval;
	bool								is_metric(d_metric_radio->isChecked());
	
	dval = d_nominal_length_edit->text().toDouble();
	
	if(!is_metric)
	{
		dval *= 25.4;
	}
	
	d_nominal_length = dval;
	
	this->Update_Expected_Length(is_metric);
}

void TTempComp::Update_Layout(void)
{
	d_dialog_layout->removeItem(d_control_layout);
	d_dialog_layout->removeWidget(d_copyright_label);
	
	if(d_display_mode == TTempComp::DISPLAY_WIDE)
	{
		d_dialog_layout->addLayout(d_control_layout,0,1,1,1);
		d_dialog_layout->addWidget(d_copyright_label,1,0,1,2);
	}
	else  // if(d_display_mode == TTempComp::DISPLAY_TALL)
	{
		d_dialog_layout->addLayout(d_control_layout,1,0,1,1);
		d_dialog_layout->addWidget(d_copyright_label,2,0,1,1);
	}
}

void TTempComp::Set_Unit_Labels(
	const bool							is_metric)
{
	if(is_metric)
	{
		d_axis_temp_units_label->setText(QStringLiteral("˚C"));
		d_axis_cte_units_label->setText(QStringLiteral("um/m/˚C"));
		d_part_temp_units_label->setText(QStringLiteral("˚C"));
		d_part_cte_units_label->setText(QStringLiteral("um/m/˚C"));
		d_nominal_length_units_label->setText(QStringLiteral("mm"));
		d_correction_units_label->setText(QStringLiteral("mm"));
		d_expected_length_units_label->setText(QStringLiteral("mm"));
	}
	else
	{
		d_axis_temp_units_label->setText(QStringLiteral("˚F"));
		d_axis_cte_units_label->setText(QStringLiteral("uIn/In/˚F"));
		d_part_temp_units_label->setText(QStringLiteral("˚F"));
		d_part_cte_units_label->setText(QStringLiteral("uIn/In/˚F"));
		d_nominal_length_units_label->setText(QStringLiteral("inch"));
		d_correction_units_label->setText(QStringLiteral("inch"));
		d_expected_length_units_label->setText(QStringLiteral("inch"));
	}
}

void TTempComp::Update_Expected_Length(
	const bool							is_metric)
{
	double								axis_temp;
	double								axis_cte;
	double								part_temp;
	double								part_cte;
	double								length;
	double								axis_change;
	double								part_change;
	double								correction;
	double								expected_length;
	
	axis_temp = d_axis_temp_edit->text().toDouble();
	axis_cte = d_axis_cte_edit->text().toDouble();
	part_temp = d_part_temp_edit->text().toDouble();
	part_cte = d_part_cte_edit->text().toDouble();
	
	length = d_nominal_length_edit->text().toDouble();

	if(is_metric)
	{
		axis_change = (axis_temp - 20.0) * length * axis_cte / 1000000.0;
		part_change = (part_temp - 20.0) * length * part_cte / 1000000.0;
	}
	else
	{
		axis_change = (axis_temp - 68.0) * length * axis_cte / 1000000.0;
		part_change = (part_temp - 68.0) * length * part_cte / 1000000.0;
	}
	
	correction = part_change - axis_change;
	expected_length = length + correction;
	
	if(is_metric)
	{
		d_correction_edit->setText(QString("%1").arg(correction,0,'f',3));
		d_expected_length_edit->setText(QString("%1").arg(expected_length,0,'f',3));
	}
	else
	{
		d_correction_edit->setText(QString("%1").arg(correction,0,'f',5));
		d_expected_length_edit->setText(QString("%1").arg(expected_length,0,'f',5));
	}
}



