/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef TEMPCOMPHEADERFILE
#define TEMPCOMPHEADERFILE

#include <QDialog>

class QDoubleValidator;
class QLabel;
class QLineEdit;
class QRadioButton;
class QSettings;
class QCloseEvent;
class QGridLayout;

class TTempComp : public QDialog
{
Q_OBJECT

public:

// CREATORS
	TTempComp(const QWidget *parent=0, Qt::WindowFlags flags=0);
	virtual ~TTempComp(void);

// ACCESSORS

// MANIPULATORS

signals:
	
protected:
	virtual void closeEvent(QCloseEvent *event);
	virtual void resizeEvent(QResizeEvent *event);

private slots:
	void Screen_Orientation_Changed(Qt::ScreenOrientation);

	void Display_Metric(bool);
	void Axis_Temp_Changed(void);
	void Axis_CTE_Changed(void);
	void Part_Temp_Changed(void);
	void Part_CTE_Changed(void);
	void Length_Changed(void);

private:
	
enum TDisplayMode
{
	DISPLAY_WIDE = 0,
	DISPLAY_TALL
};

	QDoubleValidator					*d_axis_temp_validator;
	QDoubleValidator					*d_axis_cte_validator;
	QDoubleValidator					*d_part_temp_validator;
	QDoubleValidator					*d_part_cte_validator;
	QDoubleValidator					*d_nominal_length_validator;
	QLabel								*d_axis_cte_units_label;
	QLabel								*d_axis_temp_units_label;
	QLabel								*d_correction_units_label;
	QLabel								*d_expected_length_units_label;
	QLabel								*d_icon_label;
	QLabel								*d_nominal_length_units_label;
	QLabel								*d_part_cte_units_label;
	QLabel								*d_part_temp_units_label;
	QLabel								*d_copyright_label;
	QLineEdit							*d_axis_cte_edit;
	QLineEdit							*d_axis_temp_edit;
	QLineEdit							*d_correction_edit;
	QLineEdit							*d_expected_length_edit;
	QLineEdit							*d_nominal_length_edit;
	QLineEdit							*d_part_cte_edit;
	QLineEdit							*d_part_temp_edit;
	QRadioButton						*d_imperial_radio;
	QRadioButton						*d_metric_radio;
	QSettings							*d_settings;
	QGridLayout							*d_control_layout;
	QGridLayout							*d_dialog_layout;
	
	TDisplayMode						d_display_mode;

	double								d_axis_temp;
	double								d_axis_cte;
	double								d_part_temp;
	double								d_part_cte;
	double								d_nominal_length;

	void Update_Layout(void);

	void Set_Unit_Labels(const bool is_metric);
	void Update_Expected_Length(const bool is_metric);

// NOT IMPLEMENTED
	TTempComp(const TTempComp&);
	TTempComp& operator=(const TTempComp&);
};

#endif
